@@ -4,7 +4,7 @@ Created on Sun Aug  1 13:35:28 2021
 
 @author: philippe@loco-labs.io
 
-The `observation.esconstante` module describes the constants and default values used in other modules.
+The `python.observation.esconstante` module describes the constants and default values used in other modules.
 """
 import datetime
 import math
@@ -34,7 +34,7 @@ def _classval():
 
 
 def _classESval():
-    from esobservation import LocationValue, DatationValue, PropertyValue, \
+    from observation.esobservation import LocationValue, DatationValue, PropertyValue, \
         NamedValue, ExternValue
     return {ES.obs_clsName: ExternValue,
             ES.dat_clsName: DatationValue,
@@ -83,8 +83,8 @@ class Es:
             "json_loc_name": False,  # affiche name ou instant/slot
             "json_param": False,  # ok
             "geojson": False,  # ok
-            "json_info": False,  # si True, ok pour tous les info_
-            "json_info_detail": False,
+            #"json_info": False,  # si True, ok pour tous les info_
+            #"json_info_detail": False,
             # "json_info_type"      : False,
             # "json_info_nval"      : False,
             # "json_info_box"       : False,
@@ -383,8 +383,10 @@ class Es:
         self.lencodec = "lencodec"
         self.box = "box"
         self.cat = "cat"
-        self.typecoupl = "typecoupl"
+        #self.typecoupl = "typecoupl"
         self.pname = "pname"
+        self.parent = "parent"
+        self.root = "root"
         self.typecodec = "typecodec"
         self.linkrate = "linkrate"
         self.disttomin = "disttomin"
