@@ -4,7 +4,7 @@ Created on Mon Aug  2 14:51:23 2021
 
 @author: philippe@loco-labs.io
 
-The `observation.esvalue` module contains the `observation.esvalue_base.ESValue` subclasses
+The `python.observation.esvalue` module contains the `python.observation.esvalue_base.ESValue` subclasses
 
 ESValue is build around two attributes :
 
@@ -19,7 +19,7 @@ ESValue is build around two attributes :
 
 <img src="https://loco-philippe.github.io/ES/ESValue_class.png" width="800">
 
-This module groups the classes of the objects used in the `observation.esobservation` module :
+This module groups the classes of the objects used in the `python.observation.esobservation` module :
 
 - `DatationValue`,
 - `LocationValue`,
@@ -29,7 +29,7 @@ This module groups the classes of the objects used in the `observation.esobserva
 
 and the parent class :
 
-- `observation.esvalue_base.ESValue`
+- `python.observation.esvalue_base.ESValue`
 
 Documentation is available in other pages :
     
@@ -53,9 +53,9 @@ from geopy import distance
 from copy import copy
 from openlocationcode import openlocationcode
 
-from esconstante import ES, _classval
-from esvalue_base import ESValueEncoder, ESValue
-from timeslot import TimeSlot
+from observation.esconstante import ES, _classval
+from observation.esvalue_base import ESValueEncoder, ESValue
+from observation.timeslot import TimeSlot
 
 
 class DatationValue(ESValue):   # !!! début ESValue
@@ -296,7 +296,7 @@ class LocationValue(ESValue):              # !!! début LocationValue
         if isinstance(self.value, shapely.geometry.polygon.Polygon):
             coords = [list(self.value.exterior.coords)]
         elif isinstance(self.value, shapely.geometry.point.Point):
-            coords = list(self.value.coords)[0]
+            coords = list(self.value.coords[0])
         elif isinstance(self.value, shapely.geometry.linestring.LineString):
             coords = list(self.value.coords)
         else:
