@@ -4,7 +4,7 @@ Created on Thu May 26 20:30:00 2022
 
 @author: philippe@loco-labs.io
 
-The `observation.iindex` module contains the `Iindex` class.
+The `python.observation.iindex` module contains the `Iindex` class.
 
 Documentation is available in other pages :
 
@@ -32,11 +32,11 @@ python/Examples/Iindex) are :
 # %% declarations
 from copy import copy, deepcopy
 
-from esconstante import ES
-from esvalue_base import ESValue
-from iindex_interface import IindexInterface, IindexError
-from iindex_structure import IindexStructure
-from util import util
+from observation.esconstante import ES
+from observation.esvalue_base import ESValue
+from observation.iindex_interface import IindexInterface, IindexError
+from observation.iindex_structure import IindexStructure
+from observation.util import util
 
 
 class Iindex(IindexStructure, IindexInterface):
@@ -54,6 +54,7 @@ class Iindex(IindexStructure, IindexInterface):
 
     *constructor (@classmethod)*
 
+    - `Iindex.bol`
     - `Iindex.dic`
     - `Iindex.ext`
     - `Iindex.obj`
@@ -167,6 +168,23 @@ class Iindex(IindexStructure, IindexInterface):
         if reindex:
             self.reindex()
 
+    @classmethod
+    def bol(cls, leng, notdef=None, name=None, default=True):
+        '''
+        Iindex constructor (boolean value).
+
+        *Parameters*
+
+        - **leng** : integer - length of the Iindex
+        - **notdef** : list (default None) - list of records without default value
+        - **default** : boolean (default True) - default value
+        - **name** : string (default None) - name of Iindex'''
+        values = [default] * leng
+        if notdef:
+            for item in notdef:
+                values[item] = not default        
+        return cls.ext(name=name, values=values)
+        
     @classmethod
     def dic(cls, dicvalues=None, typevalue=ES.def_clsName, fullcodec=False):
         '''
@@ -354,6 +372,7 @@ class Iindex(IindexStructure, IindexInterface):
         ''' return value item (value conversion)'''
         if isinstance(ind, tuple):
             return [copy(self.values[i]) for i in ind]
+        #return self.values[ind]
         return copy(self.values[ind])
 
     def __setitem__(self, ind, value):
@@ -368,7 +387,7 @@ class Iindex(IindexStructure, IindexInterface):
         self.reindex()
 
     def __hash__(self):
-        '''return hash(codec) + hash(keys)'''
+        '''return hash(values)'''
         return hash(tuple(self.values))
 
     def _hashe(self):
