@@ -4,8 +4,8 @@ Created on Sun Oct  2 22:24:59 2022
 
 @author: philippe@loco-labs.io
 
-The `observation.iindex_interface` module contains the `IindexInterface` class
-(`observation.iindex.Iindex` methods).
+The `python.observation.iindex_interface` module contains the `IindexInterface` class
+(`python.observation.iindex.Iindex` methods).
 """
 # %% declarations
 import json
@@ -14,9 +14,9 @@ import numpy as np
 import pandas as pd
 import cbor2
 
-from esconstante import ES
-from esvalue_base import ESValueEncoder, ESValue
-from util import util, identity
+from observation.esconstante import ES
+from observation.esvalue_base import ESValueEncoder, ESValue
+from observation.util import util, identity
 
 
 class CborDecoder(json.JSONDecoder):
@@ -123,7 +123,16 @@ class IindexInterface:
 
         *Returns* 
 
-        - **tuple** : name, typevaluedec, codec, parent, keys, isfullindex, isparent, isvar'''
+        - **tuple** : name, dtype, codec, parent, keys, isfullindex, isparent, isvar
+            name (None or string): name of the Iindex
+            dtype (None or string): type of data
+            codec (list): lilst of Iindex codec values
+            parent (int): Iindex parent or ES.nullparent
+            keys (None or list): Iindex keys
+            isfullindex (boolean): True if Iindex is full (len(keys) = len(self))
+            isparent(boolean): True if parent is >= 0
+            isvar(boolean): not used
+            '''
         if bs is None:
             return (None, None, [], ES.nullparent, None, False, False, False)
         if isinstance(bs, bytes):
@@ -450,7 +459,8 @@ class IindexInterface:
         dic['value'] = [{'record': ds[i].tolist(),
                          'codec': util.json(cod, encoded=False, typevalue=None,
                                             simpleval=simpleval, modecodec=modecodec,
-                                            untyped=option['untyped'], geojson=option['geojson'])}
+                                            untyped=option['untyped'], datetime=False,
+                                            geojson=option['geojson'])}
                         for i, cod in enumerate(self.codec)]
         return {self.name: dic}
 
