@@ -4,7 +4,7 @@ Created on Sun Oct  2 22:24:59 2022
 
 @author: philippe@loco-labs.io
 
-The `observation.ilist_analysis` module contains the `Analysis` class.
+The `python.observation.ilist_analysis` module contains the `Analysis` class.
 
 """
 
@@ -13,7 +13,7 @@ from copy import copy
 import pprint
 from collections import Counter
 
-from util import util
+from observation.util import util
 
 
 class Analysis:
@@ -38,7 +38,8 @@ class Analysis:
     The methods defined in this class are :
 
     - `Analysis.actualize`
-    - `Analysis.getinfos`
+    - `Analysis.actualize`
+    - `Analysis.check_relationship`
     - `Analysis.getmatrix`
     - `Analysis.getvarname`
     - `Analysis.getsecondary`
@@ -71,6 +72,7 @@ class Analysis:
         self.secondary = None
         self.lvarname = None
         self.partition = []
+        self.groups = []
 
     def actualize(self, partition=None):
         ''' update all data with new values of iobj
@@ -81,6 +83,7 @@ class Analysis:
         self.matrix = self._setmatrix()
         self._setinfos()
         self._setparent()
+        self._setgroups()
         self._setpartition()
         self._setinfospartition(partition)
         infosidx = [idx for idx in self.infos if idx['cat'] != 'variable']
@@ -98,6 +101,40 @@ class Analysis:
                       and self.infos[idx['parent']]['cat'] in ('primary', 'secondary')]
         self.secondary += coupledsec
 
+    def check_relationship(self, relations):
+        '''get the list of inconsistent records for each relationship defined in relations
+
+         *Parameters*
+
+        - **relations** : list of dict - list of fields with relationship property
+        
+        *Returns* : dict with for each relationship: key = pair of name, 
+        and value = list of inconsistent records'''
+        if not isinstance(relations, (list, dict)):
+            raise AnalysisError("relations is not correct")
+        if isinstance(relations, dict):
+            relations = [relations]
+        dic_res = {}
+        for field in relations:
+            if not 'relationship' in field or not 'name' in field:
+                continue
+            if not 'parent' in field['relationship'] or not 'link' in field['relationship']:
+                raise AnalysisError("relationship is not correct")
+            rel = field['relationship']['link']
+            f_parent = self.iobj.nindex(field['relationship']['parent'])
+            f_field = self.iobj.nindex(field['name'])
+            name_rel = field['name'] + ' - ' + field['relationship']['parent']
+            if f_parent is None or f_field is None:
+                raise AnalysisError("field's name are  not present in data")
+            match rel:
+                case 'derived':
+                    dic_res[name_rel] = f_parent.coupling(f_field, reindex=True)                
+                case 'coupled':
+                    dic_res[name_rel] = f_parent.coupling(f_field, derived=False, reindex=True)    
+                case _:
+                    raise AnalysisError(rel + "is not a valid relationship")
+        return dic_res          
+    
     def getinfos(self, keys=None):
         '''return attribute infos
 
@@ -126,12 +163,12 @@ class Analysis:
             self.actualize()
         if not name or not isinstance(name, list):
             return self.matrix
-        if name[0] in self.iobj.indexname:
-            ind0 = self.iobj.indexname.index(name[0])
+        if name[0] in self.iobj.lname:
+            ind0 = self.iobj.lname.index(name[0])
             if len(name) == 1:
                 return self.matrix[ind0]
-            if len(name) > 1 and name[1] in self.iobj.indexname:
-                return self.matrix[ind0][self.iobj.indexname.index(name[1])]
+            if len(name) > 1 and name[1] in self.iobj.lname:
+                return self.matrix[ind0][self.iobj.lname.index(name[1])]
         return None
 
     def getvarname(self):
@@ -158,6 +195,12 @@ class Analysis:
             self.actualize()
         return self.partition
 
+    def getgroups(self):
+        '''return attribute groups'''
+        if self.hashi != self.iobj._hashi():
+            self.actualize()
+        return self.groups
+
     def tree(self, mode='derived', width=5, lname=20, string=True):
         '''return a string with a tree of derived Iindex.
 
@@ -284,8 +327,8 @@ class Analysis:
     def _setparent(self):
         '''set parent (Iindex with minimal diff) for each Iindex'''
         # parent : min(diff) -> child
-        # distparent : min(distrate) -> diffdistparent, linkrate
-        # minparent : min(distance) -> rate, distance
+        # distparent : min(distrate) -> diffdistparent, linkrate(rateA)
+        # minparent : min(distance) -> rate(rateB), distance
         lenindex = self.iobj.lenindex
         leniobj = len(self.iobj)
         for i in range(lenindex):
@@ -362,6 +405,26 @@ class Analysis:
                     lis.append(self._dic_noeud(ch, child, lname, mode))
         return {str(n).ljust(2, '*'): lis}
 
+    def _setgroups(self):
+        '''set groups (list of crossed Iindex groups)'''
+        self.groups = []
+        crossed = {info['num'] for info in self.infos if info['crossed']}
+        remove = set()
+        for num in crossed:
+            for num2 in crossed:
+                if num != num2 and self.infos[num]['parent'] in crossed:
+                    remove.add(num)
+        crossed -= remove     
+        setcrossed = set()
+        for num in crossed:
+            info = self.infos[num]
+            if not info['name'] in setcrossed:
+                setname = {self.infos[cros]['name'] for cros in info['crossed']
+                           if cros in crossed} | {info['name']}
+                self.groups.append(setname)
+                setcrossed |= setname
+        return None
+
     def _setpartition(self):
         '''set partition (list of Iindex partitions)'''
         brother = {idx['num']: idx['crossed']
