@@ -4,8 +4,8 @@ Created on Sun Oct  2 22:24:59 2022
 
 @author: philippe@loco-labs.io
 
-The `observation.ilist_interface` module contains the `IlistInterface` class
-(`observation.ilist.Ilist` methods).
+The `python.observation.ilist_interface` module contains the `IlistInterface` class
+(`python.observation.ilist.Ilist` methods).
 """
 
 # %% declarations
@@ -19,10 +19,10 @@ import matplotlib.pyplot as plt
 from tabulate import tabulate
 import cbor2
 
-from esconstante import ES
-from iindex import Iindex
-from iindex_interface import IindexEncoder
-from util import util
+from observation.esconstante import ES
+from observation.iindex import Iindex
+from observation.iindex_interface import IindexEncoder
+from observation.util import util
 
 #import sys
 #print("In module ilist_interface sys.path[0], __package__ ==", sys.path[0], __package__)
@@ -213,11 +213,12 @@ class IlistInterface:
         - **fullvar** : boolean (default True) - if True and modecodec='optimize, 
         variable index is with a full codec
         - **geojson** : boolean (default False) - geojson for LocationValue if True
+        - **id** : integer (default None) - Observation.id if Ilist is attached to an Observation
 
         *Returns* : string, bytes or dict'''
         option = {'modecodec': 'optimize', 'encoded': False,
                   'encode_format': 'json', 'codif': ES.codeb, 'name': False,
-                  'geojson': False, 'fullvar': True} | kwargs
+                  'geojson': False, 'fullvar': True, 'id': None} | kwargs
         option2 = {'encoded': False, 'encode_format': 'json',
                    'codif': option['codif'], 'geojson': option['geojson'],
                    'modecodec': option['modecodec'], 'fullvar': option['fullvar']}
@@ -228,6 +229,17 @@ class IlistInterface:
                 if name in self.lvarname:
                     dicval['var'] = True
                 lis[name] = dicval
+        elif option['modecodec'] == 'ndjson':
+            if not option['id']:
+                raise IlistError("an  id is necessary for 'ndjson'")                
+            lis = []
+            for rec in self:
+                lis.append({ES.id: option[ES.id]} | 
+                           {name: util.json(val, encoded=False, typevalue=None,
+                                           simpleval=False, modecodec='ndjson',
+                                           untyped=False, datetime=False,
+                                           geojson=False) 
+                            for name, val in zip(self.lname, rec)})              
         else:
             indexname = [option['name'] or name != 'i' + str(i)
                          for i, name in enumerate(self.lname)]
@@ -443,7 +455,7 @@ class IlistInterface:
                 elif idx.iskeysfromderkeys(self.lindex[inf['parent']]):
                     lis.append(idx.to_obj(parent=inf['parent'],
                                           name=iname, **option2))
-                else:
+                else: # periodic derived
                     keys = idx.derkeys(self.lindex[inf['parent']])
                     lis.append(idx.to_obj(keys=keys, parent=inf['parent'],
                                           name=iname, **option2))
