@@ -4,17 +4,17 @@ Created on Sun Oct  2 22:24:59 2022
 
 @author: philippe@loco-labs.io
 
-The `observation.ilist_structure` module contains the `IlistStructure` class
-(`observation.ilist.Ilist` methods).
+The `python.observation.ilist_structure` module contains the `IlistStructure` class
+(`python.observation.ilist.Ilist` methods).
 """
 
 # %% declarations
 from copy import copy
 
-from esconstante import ES
-from iindex import Iindex
-from util import util
-from ilist_interface import IlistError
+from observation.esconstante import ES
+from observation.iindex import Iindex
+from observation.util import util
+from observation.ilist_interface import IlistError
 
 
 class IlistStructure:
@@ -253,17 +253,26 @@ class IlistStructure:
             del idx[row]
         return row
 
-    def delindex(self, indexname):
+    def delindex(self, delname=None, savename=None):
         '''remove an Iindex or a list of Iindex.
 
         *Parameters*
 
-        - **indexname** : string or list of string - name of index to remove
+        - **delname** : string or list of string - name of index to remove
+        - **savename** : string or list of string - name of index to keep
 
         *Returns* : none '''
-        if isinstance(indexname, str):
-            indexname = [indexname]
-        for idxname in indexname:
+        if not delname and not savename :
+            return
+        if isinstance(delname, str):
+            delname = [delname]
+        if isinstance(savename, str):
+            savename = [savename]
+        if delname and savename:
+            delname = [name for name in delname if not name in savename]
+        if not delname:
+            delname = [name for name in self.lname if not name in savename]
+        for idxname in delname:
             if idxname in self.lname:
                 self.lindex.pop(self.lname.index(idxname))
 
@@ -341,7 +350,9 @@ class IlistStructure:
 
         - **indexname** : list of string (default none) - name of indexes to check 
         (if None, all Iindex)
-        - **resindex** : string (default None) - Add a new index with check result
+        - **resindex** : string (default None) - Add a new index named resindex 
+        with check result (False if duplicate)
+        - **indexview** : list of str (default None) - list of fields to return
 
         *Returns* : list of int - list of rows with duplicate cod '''
         if not indexname:
